@@ -1205,6 +1205,7 @@ unsigned GB_run(GB_gameboy_t *gb)
     GB_cpu_run(gb);
     GB_clear_running_thread(gb);
     if (unlikely(gb->vblank_just_occured)) {
+        GB_update_faux_analog(gb);
         GB_debugger_handle_async_commands(gb);
         GB_set_running_thread(gb);
         GB_rewind_push(gb);
@@ -1549,7 +1550,7 @@ static void reset_ram(GB_gameboy_t *gb)
         case GB_MODEL_SGB_PAL_NO_SFC: /* Unverified */
         case GB_MODEL_SGB2:
         case GB_MODEL_SGB2_NO_SFC:
-            for (unsigned i = 0; i < 8; i++) {
+            for (unsigned i = 0; i < sizeof(gb->oam); i++) {
                 if (i & 2) {
                     gb->oam[i] = GB_random() & GB_random() & GB_random();
                 }
@@ -1557,9 +1558,6 @@ static void reset_ram(GB_gameboy_t *gb)
                     gb->oam[i] = GB_random() | GB_random() | GB_random();
                 }
             }
-            nounroll for (unsigned i = 8; i < sizeof(gb->oam); i++) {
-                gb->oam[i] = gb->oam[i - 8];
-            }
             break;
     }
     
