@@ -640,7 +640,9 @@ void GB_sgb_render(GB_gameboy_t *gb)
                     for (unsigned x = 0; x < 8; x++) {
                         *data |= pixel_to_bits[gb->sgb->screen_buffer[(tile_x + x) + (tile_y + y) * 160] & 3] >> x;
                     }
-                    *data = LE16(*data);
+                    if (gb->sgb->transfer_dest == TRANSFER_PALETTES || gb->sgb->transfer_dest == TRANSFER_BORDER_DATA) {
+                        *data = LE16(*data);
+                    }
                     data++;
                 }
             }
@@ -829,19 +831,46 @@ void GB_sgb_load_default_data(GB_gameboy_t *gb)
     gb->sgb->effective_palettes[3] = LE16(built_in_palettes[3]);
 }
 
+static double fm_sin(double phase)
+{
+#define SIN_TABLE_LENGTH 128
+    phase /= 2 * M_PI;
+    phase = fabs(phase);
+    phase -= floor(phase);
+    if (phase > 0.5) {
+        return -(fm_sin(1 - phase));
+    }
+    if (phase > 0.25) {
+        return fm_sin(0.5 - phase);
+    }
+    
+    static bool once = false;
+    static double table[SIN_TABLE_LENGTH + 1];
+    if (!once) {
+        for (unsigned i = 0; i < SIN_TABLE_LENGTH + 1; i++) {
+            table[i] = sin(i * M_PI / 2 / SIN_TABLE_LENGTH);
+        }
+        once = true;
+    }
+    
+    phase *= 4 * SIN_TABLE_LENGTH;
+    double fraction = phase - floor(phase);
+    return table[(unsigned)floor(phase)] * (1 - fraction) + table[(unsigned)ceil(phase)] * (fraction);
+}
+
 static double fm_synth(double phase)
 {
-    return (sin(phase * M_PI * 2) +
-           sin(phase * M_PI * 2 + sin(phase * M_PI * 2)) +
-           sin(phase * M_PI * 2 + sin(phase * M_PI * 3)) +
-           sin(phase * M_PI * 2 + sin(phase * M_PI * 4))) / 4;
+    return (fm_sin(phase * M_PI * 2) +
+            fm_sin(phase * M_PI * 2 + fm_sin(phase * M_PI * 2)) +
+            fm_sin(phase * M_PI * 2 + fm_sin(phase * M_PI * 3)) +
+            fm_sin(phase * M_PI * 2 + fm_sin(phase * M_PI * 4))) / 4;
 }
 
 static double fm_sweep(double phase)
 {
     double ret = 0;
     for (unsigned i = 0; i < 8; i++) {
-        ret += sin((phase * M_PI * 2 + sin(phase * M_PI * 8) / 4) * pow(1.25, i)) * (8 - i) / 36;
+        ret += fm_sin((phase * M_PI * 2 + fm_sin(phase * M_PI * 8) / 4) * pow(1.25, i)) * (8 - i) / 36;
     }
     return ret;
 }
@@ -880,14 +909,29 @@ static void render_jingle(GB_gameboy_t *gb, size_t count)
     if (sweep_cutoff_ratio > 1) {
         sweep_cutoff_ratio = 1;
     }
+
+    // Render at a lower resolution if our sample rate is too high
+    uint8_t downsample_mask = 0;
+    size_t temp_count = count;
+    while (temp_count > 2048) {
+        temp_count /= 2;
+        downsample_mask <<= 1;
+        downsample_mask |= 1;
+        sweep_phase_shift *= 2;
+        sweep_cutoff_ratio *= 2;
+    }
     
-    GB_sample_t stereo;
+    GB_sample_t stereo = {0, 0};
     for (unsigned i = 0; i < count; i++) {
+        if ((i & downsample_mask)) {
+            gb->apu_output.sample_callback(gb, &stereo);
+            continue;
+        }
         double sample = 0;
         for (signed f = 0; f < 7 && f < jingle_stage; f++) {
             sample += fm_synth(gb->sgb_intro_jingle_phases[f]) *
                       (0.75 * pow(0.5, jingle_stage - f) + 0.25) / 5.0;
-            gb->sgb_intro_jingle_phases[f] += frequencies[f] / gb->apu_output.sample_rate;
+            gb->sgb_intro_jingle_phases[f] += (frequencies[f] / gb->apu_output.sample_rate) * (downsample_mask + 1);
         }
         if (gb->sgb->intro_animation > 100) {
             sample *= pow((GB_SGB_INTRO_ANIMATION_LENGTH - gb->sgb->intro_animation) / (GB_SGB_INTRO_ANIMATION_LENGTH - 100.0), 3);
